@@ -20,14 +20,17 @@
 //! 10. Comment at the top of the file has been replaced with this documentation.
 //! 11. Some "coordinate" fields on [LevelBackgroundPosition], [EntityInstance], and [TileInstance]
 //!     have been changed from vectors to [IVec2] and [Vec2].
+//! 12. Some "color" fields on [LdtkJson], [EntityDefinition], [IntGridValueDefinition], and
+//!     [Level] have been changed from [String]s to [Color].
 
-use bevy::prelude::{IVec2, Vec2};
+use bevy::prelude::{Color, IVec2, Vec2};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 
 #[allow(unused_imports)]
 use crate::prelude::LdtkEntity;
 
+mod color;
 mod field_instance;
 
 pub use field_instance::*;
@@ -48,16 +51,16 @@ pub struct LdtkJson {
     pub backup_on_save: bool,
 
     /// Project background color
-    #[serde(rename = "bgColor")]
-    pub bg_color: String,
+    #[serde(rename = "bgColor", with = "color")]
+    pub bg_color: Color,
 
     /// Default grid size for new layers
     #[serde(rename = "defaultGridSize")]
     pub default_grid_size: i32,
 
     /// Default background color of levels
-    #[serde(rename = "defaultLevelBgColor")]
-    pub default_level_bg_color: String,
+    #[serde(rename = "defaultLevelBgColor", with = "color")]
+    pub default_level_bg_color: Color,
 
     /// Default new level height
     #[serde(rename = "defaultLevelHeight")]
@@ -184,8 +187,8 @@ pub struct Definitions {
 #[derive(PartialEq, Debug, Default, Clone, Serialize, Deserialize)]
 pub struct EntityDefinition {
     /// Base entity color
-    #[serde(rename = "color")]
-    pub color: String,
+    #[serde(rename = "color", with = "color")]
+    pub color: Color,
 
     /// Array of field definitions
     #[serde(rename = "fieldDefs")]
@@ -588,10 +591,10 @@ pub struct AutoLayerRuleDefinition {
 }
 
 /// IntGrid value definition
-#[derive(Eq, PartialEq, Debug, Default, Clone, Serialize, Deserialize)]
+#[derive(PartialEq, Debug, Default, Clone, Serialize, Deserialize)]
 pub struct IntGridValueDefinition {
-    #[serde(rename = "color")]
-    pub color: String,
+    #[serde(rename = "color", with = "color")]
+    pub color: Color,
 
     /// Unique String identifier
     #[serde(rename = "identifier")]
@@ -681,8 +684,8 @@ pub struct TilesetDefinition {
 pub struct Level {
     /// Background color of the level (same as `bgColor`, except the default value is
     /// automatically used here if its value is `null`)
-    #[serde(rename = "__bgColor")]
-    pub bg_color: String,
+    #[serde(rename = "__bgColor", with = "color")]
+    pub bg_color: Color,
 
     /// Position informations of the background image, if there is one.
     #[serde(rename = "__bgPos")]
@@ -696,8 +699,8 @@ pub struct Level {
 
     /// Background color of the level. If `null`, the project `defaultLevelBgColor` should be
     /// used.
-    #[serde(rename = "bgColor")]
-    pub level_bg_color: Option<String>,
+    #[serde(rename = "bgColor", with = "color::optional")]
+    pub level_bg_color: Option<Color>,
 
     /// Background image X pivot (0-1)
     #[serde(rename = "bgPivotX")]
