@@ -7,16 +7,13 @@ use crate::{
     },
     assets::{LdtkAsset, LdtkLevel, TilesetMap},
     components::*,
-    ldtk::{EntityDefinition, Level, TileInstance, TilesetDefinition, Type},
+    ldtk::{EntityDefinition, LayerDefinition, Level, TileInstance, TilesetDefinition, Type},
     resources::{LdtkSettings, LevelEvent, LevelSelection},
     tile_makers::*,
     utils::*,
 };
 
-use bevy::{
-    prelude::*,
-    render::{render_resource::TextureUsages, texture::DEFAULT_IMAGE_HANDLE},
-};
+use bevy::{prelude::*, render::render_resource::*};
 use bevy_ecs_tilemap::prelude::*;
 use std::collections::{HashMap, HashSet};
 
@@ -92,7 +89,6 @@ pub fn apply_level_set(
 }
 
 /// Detects [LdtkAsset] events and spawns levels as children of the [LdtkWorldBundle].
-
 #[allow(clippy::too_many_arguments)]
 pub fn process_ldtk_world(
     mut commands: Commands,
@@ -104,6 +100,7 @@ pub fn process_ldtk_world(
     level_selection: Option<Res<LevelSelection>>,
     ldtk_assets: Res<Assets<LdtkAsset>>,
     ldtk_settings: Res<LdtkSettings>,
+    mut clear_color: Option<ResMut<ClearColor>>,
     layer_query: Query<&Layer>,
     chunk_query: Query<&Chunk>,
 ) {
@@ -154,6 +151,14 @@ pub fn process_ldtk_world(
             }
 
             if let Some(ldtk_asset) = ldtk_assets.get(ldtk_handle) {
+                if ldtk_settings.set_clear_color {
+                    if let Some(clear_color) = &mut clear_color {
+                        clear_color.0 = ldtk_asset.project.bg_color;
+                    } else {
+                        commands.insert_resource(ClearColor(ldtk_asset.project.bg_color));
+                    }
+                }
+
                 if let Some(level_selection) = &level_selection {
                     if let Some(level) = ldtk_asset.get_level(level_selection) {
                         level_set.uids.clear();
@@ -253,6 +258,7 @@ fn clear_map(
 pub fn process_ldtk_levels(
     mut commands: Commands,
     asset_server: Res<AssetServer>,
+    mut images: ResMut<Assets<Image>>,
     mut meshes: ResMut<Assets<Mesh>>,
     mut texture_atlases: ResMut<Assets<TextureAtlas>>,
     ldtk_assets: Res<Assets<LdtkAsset>>,
@@ -281,6 +287,9 @@ pub fn process_ldtk_levels(
                 let entity_definition_map =
                     create_entity_definition_map(&ldtk_asset.project.defs.entities);
 
+                let layer_definition_map =
+                    create_layer_definition_map(&ldtk_asset.project.defs.layers);
+
                 let worldly_set = worldly_query.iter().cloned().collect();
 
                 if let Some(level) = level_assets.get(level_handle) {
@@ -288,11 +297,13 @@ pub fn process_ldtk_levels(
                         &level.level,
                         &mut commands,
                         &asset_server,
+                        &mut images,
                         &mut texture_atlases,
                         &mut meshes,
                         &ldtk_entity_map,
                         &ldtk_int_cell_map,
                         &entity_definition_map,
+                        &layer_definition_map,
                         &ldtk_asset.tileset_map,
                         &tileset_definition_map,
                         worldly_set,
@@ -310,11 +321,13 @@ fn spawn_level(
     level: &Level,
     commands: &mut Commands,
     asset_server: &AssetServer,
+    images: &mut Assets<Image>,
     texture_atlases: &mut Assets<TextureAtlas>,
     meshes: &mut ResMut<Assets<Mesh>>,
     ldtk_entity_map: &LdtkEntityMap,
     ldtk_int_cell_map: &LdtkIntCellMap,
     entity_definition_map: &HashMap<i32, &EntityDefinition>,
+    layer_definition_map: &HashMap<i32, &LayerDefinition>,
     tileset_map: &TilesetMap,
     tileset_definition_map: &HashMap<i32, &TilesetDefinition>,
     worldly_set: HashSet<Worldly>,
@@ -324,6 +337,54 @@ fn spawn_level(
 
     if let Some(layer_instances) = &level.layer_instances {
         let mut layer_id = 0;
+
+        // creating an image to use for the background color, and for intgrid colors
+        let mut white_image = Image::new_fill(
+            Extent3d {
+                width: level.px_wid as u32,
+                height: level.px_hei as u32,
+                depth_or_array_layers: 1,
+            },
+            TextureDimension::D2,
+            &[255, 255, 255, 255],
+            TextureFormat::Rgba8UnormSrgb,
+        );
+        white_image.texture_descriptor.usage =
+            TextureUsages::TEXTURE_BINDING | TextureUsages::COPY_SRC | TextureUsages::COPY_DST;
+
+        let white_image_handle = images.add(white_image);
+
+        {
+            let settings = LayerSettings::new(
+                MapSize(1, 1),
+                ChunkSize(1, 1),
+                TileSize(level.px_wid as f32, level.px_hei as f32),
+                TextureSize(level.px_wid as f32, level.px_hei as f32),
+            );
+
+            let (mut layer_builder, layer_entity) =
+                LayerBuilder::<TileBundle>::new(commands, settings, map.id, layer_id);
+
+            match layer_builder.set_tile(
+                TilePos(0, 0),
+                TileBundle {
+                    tile: Tile {
+                        color: level.bg_color,
+                        ..Default::default()
+                    },
+                    ..Default::default()
+                },
+            ) {
+                Ok(()) => (),
+                Err(_) => warn!("Encountered error when setting background tile"),
+            }
+
+            let layer_bundle = layer_builder.build(commands, meshes, white_image_handle.clone());
+            commands.entity(layer_entity).insert_bundle(layer_bundle);
+            map.add_layer(commands, layer_id, layer_entity);
+            layer_id += 1;
+        }
+
         for layer_instance in layer_instances.iter().rev() {
             match layer_instance.layer_instance_type {
                 Type::Entities => {
@@ -415,7 +476,10 @@ fn spawn_level(
                             tileset_definition.px_wid as f32,
                             tileset_definition.px_hei as f32,
                         ),
-                        None => TextureSize(0., 0.),
+                        None => TextureSize(
+                            layer_instance.grid_size as f32,
+                            layer_instance.grid_size as f32,
+                        ),
                     };
 
                     let mut settings =
@@ -424,13 +488,18 @@ fn spawn_level(
                     if let Some(tileset_definition) = tileset_definition {
                         settings.grid_size = Vec2::splat(layer_instance.grid_size as f32);
                         if tileset_definition.spacing != 0 {
-                            warn!(
-                                "Tile spacing currently not supported for AutoTile and Tile layers"
-                            );
+                            #[cfg(not(feature = "atlas"))]
+                            {
+                                warn!(
+                                    "Tile spacing on Tile and AutoTile layers requires the \"atlas\" feature"
+                                );
+                            }
 
-                            // This causes a crash after bevy_ecs_tilemap switched to texture
-                            // arrays
-                            //settings.tile_spacing = Vec2::splat(tileset_definition.spacing as f32);
+                            #[cfg(feature = "atlas")]
+                            {
+                                settings.tile_spacing =
+                                    Vec2::splat(tileset_definition.spacing as f32);
+                            }
                         }
                     }
 
@@ -446,7 +515,7 @@ fn spawn_level(
                         Some(tileset_definition) => {
                             tileset_map.get(&tileset_definition.uid).unwrap().clone()
                         }
-                        None => DEFAULT_IMAGE_HANDLE.typed(),
+                        None => white_image_handle.clone(),
                     };
 
                     let mut grid_tiles = layer_instance.grid_tiles.clone();
@@ -458,12 +527,13 @@ fn spawn_level(
                             // LayerBuilder::new_batch().
                             // So, the actual LayerBuilder usage diverges greatly here
 
-                            let (mut layer_builder, layer_entity) = LayerBuilder::<TileBundle>::new(
-                                commands,
-                                settings,
-                                map.id,
-                                layer_id as u16,
-                            );
+                            let (mut layer_builder, layer_entity) =
+                                LayerBuilder::<TileGridBundle>::new(
+                                    commands,
+                                    settings,
+                                    map.id,
+                                    layer_id as u16,
+                                );
 
                             match tileset_definition {
                                 Some(_) => {
@@ -479,13 +549,20 @@ fn spawn_level(
                                     );
                                 }
                                 None => {
+                                    let int_grid_value_defs = &layer_definition_map
+                                        .get(&layer_instance.layer_def_uid)
+                                        .expect("Encountered layer without definition")
+                                        .int_grid_values;
+
                                     set_all_tiles_with_func(
                                         &mut layer_builder,
-                                        tile_pos_to_tile_bundle_if_int_grid_nonzero_maker(
-                                            tile_pos_to_invisible_tile,
-                                            &layer_instance.int_grid_csv,
-                                            layer_instance.c_wid,
-                                            layer_instance.c_hei,
+                                        tile_pos_to_tile_bundle_maker(
+                                            tile_pos_to_int_grid_colored_tile_maker(
+                                                &layer_instance.int_grid_csv,
+                                                int_grid_value_defs,
+                                                layer_instance.c_wid,
+                                                layer_instance.c_hei,
+                                            ),
                                         ),
                                     );
                                 }
@@ -552,7 +629,7 @@ fn spawn_level(
                                 grid_tiles,
                             );
 
-                            LayerBuilder::<TileBundle>::new_batch(
+                            LayerBuilder::<TileGridBundle>::new_batch(
                                 commands,
                                 settings,
                                 meshes,
