@@ -1,7 +1,10 @@
 //! Utility functions used internally by the plugin that have been exposed to the public api.
 
 #[allow(unused_imports)]
-use crate::components::IntGridCell;
+use crate::{
+    app::LdtkEntity,
+    components::{GridCoords, IntGridCell},
+};
 
 use crate::ldtk::*;
 use bevy::prelude::*;
@@ -51,6 +54,13 @@ pub fn create_entity_definition_map(
     entity_definitions.iter().map(|e| (e.uid, e)).collect()
 }
 
+/// Simple conversion from a list of [LayerDefinition]s to a map using their Uids as the keys.
+pub fn create_layer_definition_map(
+    layer_definitions: &[LayerDefinition],
+) -> HashMap<i32, &LayerDefinition> {
+    layer_definitions.iter().map(|l| (l.uid, l)).collect()
+}
+
 /// Performs [EntityInstance] to [Transform] conversion
 ///
 /// The `entity_definition_map` should be a map of [EntityDefinition] uids to [EntityDefinition]s.
@@ -112,6 +122,16 @@ pub fn ldtk_grid_coords_to_tile_pos(ldtk_coords: IVec2, ldtk_grid_height: i32) -
     TilePos(tile_coords.x, tile_coords.y)
 }
 
+/// Performs LDtk grid coordinate to [GridCoords] conversion.
+///
+/// This conversion is performed so that both the LDtk grid coords and the resulting [GridCoords]
+/// refer to the same tile.
+/// This is different from them referring to the same position in space, because the tile is
+/// referenced by its top-left corner in LDtk, and by its bottom-left corner with [GridCoords].
+pub fn ldtk_grid_coords_to_grid_coords(ldtk_coords: IVec2, ldtk_grid_height: i32) -> GridCoords {
+    ldtk_coord_conversion_origin_adjusted(ldtk_coords, ldtk_grid_height).into()
+}
+
 /// Performs [TilePos] to LDtk grid coordinate conversion.
 ///
 /// This conversion is performed so that both the [TilePos] and the resulting LDtk grid coords
@@ -236,6 +256,62 @@ where
     try_each_optional_permutation(a, b, |x, y| map.get(&(x, y))).unwrap_or(default)
 }
 
+/// Creates a [SpriteSheetBundle] from the entity information available to the
+/// [LdtkEntity::bundle_entity] method.
+///
+/// Used for the `#[sprite_sheet_bundle]` attribute macro for `#[derive(LdtkEntity)]`.
+/// See [LdtkEntity#sprite_sheet_bundle] for more info.
+pub fn sprite_sheet_bundle_from_entity_info(
+    entity_instance: &EntityInstance,
+    tileset: Option<&Handle<Image>>,
+    tileset_definition: Option<&TilesetDefinition>,
+    texture_atlases: &mut Assets<TextureAtlas>,
+) -> SpriteSheetBundle {
+    match (tileset, &entity_instance.tile, tileset_definition) {
+        (Some(tileset), Some(tile), Some(tileset_definition)) => SpriteSheetBundle {
+            texture_atlas: texture_atlases.add(TextureAtlas::from_grid_with_padding(
+                tileset.clone(),
+                Vec2::new(tile.src_rect[2] as f32, tile.src_rect[3] as f32),
+                tileset_definition.c_wid as usize,
+                tileset_definition.c_hei as usize,
+                Vec2::splat(tileset_definition.spacing as f32),
+            )),
+            sprite: TextureAtlasSprite {
+                index: (tile.src_rect[1] / (tile.src_rect[3] + tileset_definition.spacing))
+                    as usize
+                    * tileset_definition.c_wid as usize
+                    + (tile.src_rect[0] / (tile.src_rect[2] + tileset_definition.spacing)) as usize,
+                ..Default::default()
+            },
+            ..Default::default()
+        },
+        _ => {
+            warn!("EntityInstance needs a tile, an associated tileset, and an associated tileset definition to be bundled as a SpriteSheetBundle");
+            SpriteSheetBundle::default()
+        }
+    }
+}
+
+/// Creates a [SpriteBundle] from the entity information available to the
+/// [LdtkEntity::bundle_entity] method.
+///
+/// Used for the `#[sprite_bundle]` attribute macro for `#[derive(LdtkEntity)]`.
+/// See [LdtkEntity#sprite_bundle] for more info.
+pub fn sprite_bundle_from_entity_info(tileset: Option<&Handle<Image>>) -> SpriteBundle {
+    let tileset = match tileset {
+        Some(tileset) => tileset.clone(),
+        None => {
+            warn!("EntityInstance needs a tileset to be bundled as a SpriteBundle");
+            return SpriteBundle::default();
+        }
+    };
+
+    SpriteBundle {
+        texture: tileset,
+        ..Default::default()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
